require_relative 'game'
require_relative 'treasure_trove'
require_relative 'playable'

module StudioGame
	class Player
		include Playable

		attr_accessor 	:name
		attr_accessor 	:health

		def initialize(name, health=100)
			@name = name.capitalize
			@health = health
			@found_treasures = Hash.new(0)
		end

		def to_csv
			"#{@name}, #{@health}"
		end

		def self.from_csv(string)
	      name, health = string.split(',')    
	      new(name, Integer(health))
	    end 

		def each_found_treasure
			@found_treasures.each do |name, points|
				treasure = Treasure.new(name, points)
				yield treasure
			end
		end

		def found_treasure(treasure)
			@found_treasures[treasure.name] += treasure.points
			puts "#{@name}'s treasures: #{@found_treasures}"
			puts "#{@name} found a #{treasure.name} worth #{treasure.points} points."
		end

		def points
			@found_treasures.values.reduce(0, :+)	
		end

		def name=(new_name)
			@name = new_name.capitalize
		end

		def score
			@health + points
		end

		def to_s
			"I'm #{@name} with health = #{@health}, points = #{points}, and score of #{score}."
		end

		def <=>(other_player)
			other_player.health <=> @health
		end
	end
end

if __FILE__ == $0
	player = Player.new("moe", 150)
	puts player.name
	puts player.health
	player.w00t
	puts player.health
	player.blam
	puts player.health
end

